@@ -8,30 +8,20 @@
 use std::rc::Rc;
 use std::cell::RefCell;
 //use std::ops::Index;
+use ::rand::prelude::StdRng;
 
 #[cfg(feature = "use-serde")]
 use serde::{Serialize, Deserialize};
 
 use std::fmt;
 
-pub mod gen_tensor;
-#[cfg(feature = "use-cuda")]
-pub mod cuda_tensor;
-#[cfg(feature = "use-cuda")]
-pub mod cuda_helper;
-pub mod blas;
-pub mod typed_tensor;
-pub mod compare_tensor;
-pub mod elemwise;
-pub mod index_slicing;
-pub mod convolution;
-pub mod reduction;
 
 
-use typed_tensor::TypedTensor;
-use gen_tensor::GenTensor;
-//use quaternion::Quaternion;
 
+use super::typed_tensor::TypedTensor;
+use crate::tensor_impl::gen_tensor::GenTensor;
+
+/// 2-to-1
 macro_rules! tensor_method {
     ($a:ident) => {
         pub fn $a(&self, o: &Tensor) -> Tensor {
@@ -42,6 +32,17 @@ macro_rules! tensor_method {
     }
 }
 
+/// 2-to-1option
+macro_rules! tensor_method_2_to_1option {
+    ($a:ident) => {
+        pub fn $a(&self, o: &Tensor) -> Option<Tensor> {
+            self.v.borrow().$a(&o.v.borrow()).map(|v| Tensor {
+                v: Rc::new(RefCell::new(v))})            
+        }
+    }
+}
+
+/// 1-to-other
 macro_rules! tensor_method_single_same_return {
     ($a:ident, $b:ty) => {
         pub fn $a(&self) -> $b {
@@ -50,6 +51,7 @@ macro_rules! tensor_method_single_same_return {
     }
 }
 
+/// 1-to-1
 macro_rules! tensor_method_single_tensor_return {
     ($a:ident) => {
         pub fn $a(&self) -> Tensor {
@@ -60,11 +62,65 @@ macro_rules! tensor_method_single_tensor_return {
     }
 }
 
+/// 1-to-1option
+macro_rules! tensor_method_1_option_tensor_return {
+    ($a:ident) => {
+        pub fn $a(&self) -> Option<Tensor> {
+            let r = self.v.borrow().$a();
+            r.map(|r1| Tensor {
+                v: Rc::new(RefCell::new(r1)),
+            })            
+        }
+    }
+}
+
+/// 1-to-2option
+macro_rules! tensor_method_2_option_tensor_return {
+    ($a:ident) => {
+        pub fn $a(&self) -> Option<[Tensor; 2]> {
+            let r = self.v.borrow().$a();
+            r.map(|[r1, r2]| [Tensor {
+                v: Rc::new(RefCell::new(r1)),},
+                              Tensor {
+                                  v: Rc::new(RefCell::new(r2)),
+                              }])
+        }
+    }
+}
+
+/// 1-to-3option
+macro_rules! tensor_method_3_option_tensor_return {
+    ($a:ident) => {
+        pub fn $a(&self) -> Option<[Tensor; 3]> {
+            let r = self.v.borrow().$a();
+            r.map(|[r1, r2, r3]| [
+                Tensor {
+                        v: Rc::new(RefCell::new(r1)),},
+                          Tensor {
+                              v: Rc::new(RefCell::new(r2)),
+                          },
+                          Tensor {
+                              v: Rc::new(RefCell::new(r3)),
+                          }
+            ])
+        }
+    }
+}
+
 
 
 pub struct Tensor {
     v: Rc<RefCell<TypedTensor>>,
 }
+
+impl Default for Tensor {
+    fn default() -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::new())),
+        }
+    }
+}
+
 impl Tensor {
     pub fn new() -> Tensor {
         Tensor {
@@ -72,6 +128,20 @@ impl Tensor {
         }
     }
 
+    pub fn data_copy(&self, o: &Tensor) {
+        self.v.borrow_mut().data_copy(&o.v.borrow());
+    }
+
+    pub fn swap(&self, o: &Tensor) {
+        self.v.swap(&o.v);
+    }
+
+    pub fn ref_copy(&self) -> Tensor {
+        Tensor {
+            v: self.v.clone(),
+        }
+    }
+
     /// Right most is the continous indexing,
     /// This method convert continuous index to index along each dimension.
     pub fn index2dimpos(&self, index: usize) -> Vec::<usize> {
@@ -95,6 +165,9 @@ impl Tensor {
     pub fn get_scale_f32(&self) -> f32 {
         self.v.borrow().get_scale_f32()
     }
+    pub fn get_scale_f64(&self) -> f64 {
+        self.v.borrow().get_scale_f64()
+    }
 
     tensor_method_single_tensor_return!(get_n);
     tensor_method_single_tensor_return!(get_c);
@@ -161,7 +234,7 @@ impl Tensor {
     }
 
     
-    pub fn from_record(&self, row: usize, record: &[f32]) -> Result<(), ()> {
+    pub fn from_record(&self, row: usize, record: &[f32]) -> Result<(), &'static str> {
         self.v.borrow_mut().from_record(row, record)
     }
     pub fn get_f32(&self, o: &[usize]) -> f32 {
@@ -171,20 +244,39 @@ impl Tensor {
         self.v.borrow_mut().set_f32(o, v);
     }
 
-    pub fn swap(&self, o: Tensor) {
-        self.v.swap(&o.v);
+    pub fn get_f64(&self, o: &[usize]) -> f64 {
+        self.v.borrow().get_f64(o)
     }
+    pub fn set_f64(&mut self, o: &[usize], v: f64) {
+        self.v.borrow_mut().set_f64(o, v);
+    }
+
+
     
     /// Returns a tensor of size size filled with fill_value.
-    pub fn fill(size: &[usize], fill_value: f32) -> Tensor {
+    pub fn fill(size: &[usize], fill_value: &Tensor) -> Tensor {
         Tensor {
-            v: Rc::new(RefCell::new(TypedTensor::fill(size, fill_value))),
+            v: Rc::new(RefCell::new(TypedTensor::fill(size, &fill_value.v.borrow()))),
         }
     }
+    pub fn fill_f32(size: &[usize], fill_value: f32) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::fill_f32(size, fill_value))),
+        }
+    }
+    pub fn fill_f64(size: &[usize], fill_value: f64) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::fill_f64(size, fill_value))),
+        }
+    }
+    
     // zeros
     pub fn zeros(dim: &[usize]) -> Tensor {
         Tensor {
-            v: Rc::new(RefCell::new(TypedTensor::zeros(dim))),
+            #[cfg(feature = "use-f64")]
+            v: Rc::new(RefCell::new(TypedTensor::zeros_f64(dim))),
+            #[cfg(feature = "use-f32")]
+            v: Rc::new(RefCell::new(TypedTensor::zeros_f32(dim))),
         }
     }
     // zeros_like
@@ -192,19 +284,21 @@ impl Tensor {
     // ones
     pub fn ones(dim: &[usize]) -> Tensor {
         Tensor {
-            v: Rc::new(RefCell::new(TypedTensor::ones(dim))),
+            #[cfg(feature = "use-f64")]
+            v: Rc::new(RefCell::new(TypedTensor::ones_f64(dim))),
+            #[cfg(feature = "use-f32")]
+            v: Rc::new(RefCell::new(TypedTensor::ones_f32(dim))),
         }
     }
     // ones_like
     tensor_method_single_tensor_return!(ones_like);
     // range
     pub fn range(start: f32, end: f32, step: Option<f32>) -> Tensor {
-        let real_step;
-        if let Some(v) = step {
-            real_step = v;
+        let real_step = if let Some(v) = step {
+            v
         } else {
-            real_step = 1.;
-        }
+            1.
+        };
 
         let mut value = start;
         let mut index = 0;
@@ -215,7 +309,7 @@ impl Tensor {
             index += 1;
         }
         
-        Tensor::from_vec_f32(&data, &vec![index])
+        Tensor::from_vec_f32(&data, &[index])
     }
     // linspace
     pub fn linspace(start: f32, end: f32, steps: usize) -> Tensor {
@@ -230,22 +324,22 @@ impl Tensor {
             index += 1;
         }
         
-        Tensor::from_vec_f32(&data, &vec![index])
+        Tensor::from_vec_f32(&data, &[index])
     }
     // logspace
     pub fn logspace(start: f32, end: f32, steps: usize, base: f32) -> Tensor {
         let linspace_data = Tensor::linspace(start, end, steps);
         let mut ret_data = Vec::new();
         for i in 0..linspace_data.numel() {
-            ret_data.push(base.powf(linspace_data.get_f32(&vec![i])));
+            ret_data.push(base.powf(linspace_data.get_f32(&[i])));
         }
-        Tensor::from_vec_f32(&ret_data, &vec![ret_data.len()])
+        Tensor::from_vec_f32(&ret_data, &[ret_data.len()])
     }
     // eye
     pub fn eye(n: usize, m: usize) -> Tensor {
-        let ret = Tensor::empty(&vec![n, m]);
+        let ret = Tensor::zeros(&[n, m]);
         for i in 0..n.min(m) {
-            ret.v.borrow_mut().set_f32(&vec![i, i], 1.);
+            ret.v.borrow_mut().set_f32(&[i, i], 1.);
         }
         ret
     }
@@ -253,32 +347,38 @@ impl Tensor {
     pub fn empty(shape: &[usize]) -> Tensor {
         for i in shape {
             if *i == 0 {
-                println!("");
+                println!("empty: shape with zeros in it.");
             }
         }
         Tensor {
-            v: Rc::new(RefCell::new(TypedTensor::empty(shape))),
+            #[cfg(feature = "use-f64")]
+            v: Rc::new(RefCell::new(TypedTensor::zeros_f64(shape))),
+            #[cfg(feature = "use-f32")]
+            v: Rc::new(RefCell::new(TypedTensor::zeros_f32(shape))),
         }
     }
 
     
     // Indexing, Slicing, Joining, Mutating Ops
-    pub fn cat(&self, tensors: &[&Tensor], dim: usize) -> Tensor {
+    pub fn cat(&self, tensors: &[Tensor], dim: usize) -> Tensor {
         let mut concrete_tensor = Vec::new();
         
         for i in tensors {
             concrete_tensor.push(i.v.borrow().clone());
         }
-        let mut converted_tensor = Vec::new();
-        for i in &concrete_tensor {
-            converted_tensor.push(i);
-        }
         Tensor {
-            v: Rc::new(RefCell::new(self.v.borrow().cat(&converted_tensor[..], dim))),
+            v: Rc::new(RefCell::new(self.v.borrow().cat(&concrete_tensor, dim))),
         }
     }
-    pub fn chunk() {
-        unimplemented!();
+    pub fn chunk(&self, chunks: usize, dim: usize) -> Vec<Tensor> {
+        let mut result = self.v.borrow().chunk(chunks, dim);
+        let mut ret = Vec::new();
+        for i in result.drain(..) {
+            ret.push(Tensor {
+                v: Rc::new(RefCell::new(i))
+            });
+        }
+        ret
     }
     pub fn gather(&self, dim: usize, index: &Tensor) -> Tensor {
         Tensor {
@@ -290,6 +390,11 @@ impl Tensor {
             v: Rc::new(RefCell::new(self.v.borrow().index_select(dim, &index.v.borrow()))),
         }
     }
+    pub fn index_exclude(&self, dim: usize, index: &Tensor) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(self.v.borrow().index_exclude(dim, &index.v.borrow()))),
+        }
+    }
     pub fn masked_select() {
         unimplemented!();
     }
@@ -319,14 +424,25 @@ impl Tensor {
             v: Rc::new(RefCell::new(self.v.borrow().squeeze(dim))),
         }
     }
-    pub fn stack() {
-        unimplemented!();
+    pub fn stack(&self, tensors: &[Tensor], dim: usize) -> Tensor {
+        let mut concrete_tensor = Vec::new();
+        
+        for i in tensors {
+            concrete_tensor.push(i.v.borrow().clone());
+        }
+        Tensor {
+            v: Rc::new(RefCell::new(self.v.borrow().stack(&concrete_tensor, dim))),
+        }
     }
-    pub fn t() {
-        unimplemented!();
+    pub fn t(&self) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(self.v.borrow().t()))
+        }
     }
-    pub fn take() {
-        unimplemented!();
+    pub fn take(&self, index: &[usize]) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(self.v.borrow().take(index)))
+        }
     }
     pub fn transpose() {
         unimplemented!();
@@ -382,7 +498,7 @@ impl Tensor {
     tensor_method_single_tensor_return!(floor);
     tensor_method_single_tensor_return!(frac);
     // lerp
-    pub fn lerp(&self, end: &Tensor, weight: f32) -> Tensor {
+    pub fn lerp(&self, end: &Tensor, weight: &Tensor) -> Tensor {
         self.add(&Tensor::fill(&self.size(), weight).mul(&end.sub(self)))
     }
     tensor_method_single_tensor_return!(log);
@@ -480,6 +596,7 @@ impl Tensor {
         }
     }
 
+    // linalg
     pub fn normalize(&self, mean: &[f32], std: &[f32]) -> Tensor {
         if self.size().len() != 2 {
             panic!("fn normalize is for two-dimensional data.");
@@ -493,13 +610,12 @@ impl Tensor {
         }
         
         let data_mean = self.mean(Some(&[0]), false);
-        let tmp1 = self.sub(&data_mean).add(&Tensor::from_vec_f32(mean, &vec![width]));
+        let tmp1 = self.sub(&data_mean).add(&Tensor::from_vec_f32(mean, &[width]));
         //let tmp1 = Tensor::from_vec_f32(mean, &vec![width]).sub(&data_mean).add(self);
 
         let data_std = tmp1.std(Some(&[0]), false);
         //println!("data_std: {:?}, tmp1: {:?}", data_std, tmp1);
-        let tmp2 = tmp1.div(&data_std);
-        tmp2
+        tmp1.div(&data_std)
     }
     pub fn normalize_unit(&self) -> Tensor {
         if self.size().len() != 2 {
@@ -509,6 +625,16 @@ impl Tensor {
                        &vec![1. ; self.size()[self.size().len()-1]])
     }
 
+    tensor_method_2_option_tensor_return!(lu);
+    tensor_method_2_to_1option!(lu_solve);
+    tensor_method_2_option_tensor_return!(qr);
+    tensor_method_2_option_tensor_return!(eigen);
+    tensor_method_1_option_tensor_return!(cholesky);
+    tensor_method_1_option_tensor_return!(det);
+    tensor_method_3_option_tensor_return!(svd);
+    tensor_method_1_option_tensor_return!(inv);
+    tensor_method_single_tensor_return!(pinv);
+
 
     // Comparison Ops
     tensor_method!(all_close);
@@ -521,6 +647,44 @@ impl Tensor {
     tensor_method!(min_pair);
     tensor_method!(ne);
 
+    // rand
+    pub fn rand_usize(rng: &mut StdRng,
+                      dim: &[usize],
+                      left: usize, right: usize) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::rand_usize(rng, dim, left, right))),
+        }
+    }
+    pub fn normal_f64(rng: &mut StdRng,
+                  dim: &[usize],
+                  mean: f64, std: f64) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::normal_f64(rng, dim, mean, std))),
+        }
+    }
+    pub fn normal_f32(rng: &mut StdRng,
+                  dim: &[usize],
+                  mean: f32, std: f32) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::normal_f32(rng, dim, mean, std))),
+        }
+    }
+    pub fn uniform_f64(rng: &mut StdRng,
+                   dim: &[usize],
+                   from: f64, to: f64) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::uniform_f64(rng, dim, from, to)))
+        }
+    }
+    pub fn uniform_f32(rng: &mut StdRng,
+                   dim: &[usize],
+                   from: f32, to: f32) -> Tensor {
+        Tensor {
+            v: Rc::new(RefCell::new(TypedTensor::uniform_f32(rng, dim, from, to)))
+        }
+    }
+    
+
     // conv ops
     pub fn conv2d(&self, o: &Tensor,
                   stride: (usize, usize),
@@ -553,7 +717,7 @@ impl fmt::Display for Tensor {
 }
 impl fmt::Debug for Tensor {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "({}, )", self.v.borrow())
+        write!(f, "({:?}, )", self.v.borrow())
     }
 }
 impl PartialEq for Tensor {
